# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from collections.abc import MutableMapping
from io import IOBase
import json
from typing import Any, Callable, IO, Optional, TypeVar, Union, overload

from azure.core import AsyncPipelineClient
from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    StreamClosedError,
    StreamConsumedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.rest import AsyncHttpResponse, HttpRequest
from azure.core.tracing.decorator_async import distributed_trace_async
from azure.core.utils import case_insensitive_dict

from ... import models as _models
from ..._utils.model_base import SdkJSONEncoder, _deserialize
from ..._utils.serialization import Deserializer, Serializer
from ...operations._operations import (
    build_attestation_attest_azure_guest_request,
    build_attestation_attest_open_enclave_request,
    build_attestation_attest_sev_snp_vm_request,
    build_attestation_attest_sgx_enclave_request,
    build_attestation_attest_tdx_vm_request,
    build_attestation_attest_tpm_request,
    build_metadata_configuration_get_request,
    build_policy_certificates_add_request,
    build_policy_certificates_get_request,
    build_policy_certificates_remove_request,
    build_policy_get_request,
    build_policy_reset_request,
    build_policy_set_request,
    build_signing_certificates_get_request,
    build_tcb_baselines_get_request,
)
from .._configuration import AttestationClientConfiguration

T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, dict[str, Any]], Any]]
JSON = MutableMapping[str, Any]
_Unset: Any = object()


class PolicyOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.security.attestation.aio.AttestationClient`'s
        :attr:`policy` attribute.
    """

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: AsyncPipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: AttestationClientConfiguration = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace_async
    async def get(self, attestation_type: Union[str, _models.AttestationType], **kwargs: Any) -> _models.PolicyResponse:
        """Retrieves the current policy for an attestation type.

        Retrieves the current policy for an attestation type.

        :param attestation_type: Specifies the trusted execution environment to be used to validate the
         evidence. Known values are: "SgxEnclave", "OpenEnclave", "AzureGuest", "SevSnpVm", "Tpm", and
         "TdxVm". Required.
        :type attestation_type: str or ~azure.security.attestation.models.AttestationType
        :return: PolicyResponse. The PolicyResponse is compatible with MutableMapping
        :rtype: ~azure.security.attestation.models.PolicyResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[_models.PolicyResponse] = kwargs.pop("cls", None)

        _request = build_policy_get_request(
            attestation_type=attestation_type,
            api_version=self._config.api_version,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    await response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if _stream:
            deserialized = response.iter_bytes()
        else:
            deserialized = _deserialize(_models.PolicyResponse, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace_async
    async def set(
        self, attestation_type: Union[str, _models.AttestationType], new_attestation_policy: str, **kwargs: Any
    ) -> _models.PolicyResponse:
        """Sets the policy for a given attestation type.

        Sets the policy for a given attestation type.

        :param attestation_type: Specifies the trusted execution environment to be used to validate the
         evidence. Known values are: "SgxEnclave", "OpenEnclave", "AzureGuest", "SevSnpVm", "Tpm", and
         "TdxVm". Required.
        :type attestation_type: str or ~azure.security.attestation.models.AttestationType
        :param new_attestation_policy: JWT Expressing the new policy whose body is a
         StoredAttestationPolicy object. Required.
        :type new_attestation_policy: str
        :return: PolicyResponse. The PolicyResponse is compatible with MutableMapping
        :rtype: ~azure.security.attestation.models.PolicyResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: str = kwargs.pop("content_type", _headers.pop("Content-Type", "text/plain"))
        cls: ClsType[_models.PolicyResponse] = kwargs.pop("cls", None)

        _content = new_attestation_policy

        _request = build_policy_set_request(
            attestation_type=attestation_type,
            content_type=content_type,
            api_version=self._config.api_version,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    await response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if _stream:
            deserialized = response.iter_bytes()
        else:
            deserialized = _deserialize(_models.PolicyResponse, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace_async
    async def reset(
        self, attestation_type: Union[str, _models.AttestationType], policy_jws: str, **kwargs: Any
    ) -> _models.PolicyResponse:
        """Resets the attestation policy for the specified tenant and reverts to the
        default policy.

        Resets the attestation policy for the specified tenant and reverts to the
        default policy.

        :param attestation_type: Specifies the trusted execution environment to be used to validate the
         evidence. Known values are: "SgxEnclave", "OpenEnclave", "AzureGuest", "SevSnpVm", "Tpm", and
         "TdxVm". Required.
        :type attestation_type: str or ~azure.security.attestation.models.AttestationType
        :param policy_jws: JSON Web Signature with an empty policy document. Required.
        :type policy_jws: str
        :return: PolicyResponse. The PolicyResponse is compatible with MutableMapping
        :rtype: ~azure.security.attestation.models.PolicyResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: str = kwargs.pop("content_type", _headers.pop("Content-Type", "text/plain"))
        cls: ClsType[_models.PolicyResponse] = kwargs.pop("cls", None)

        _content = policy_jws

        _request = build_policy_reset_request(
            attestation_type=attestation_type,
            content_type=content_type,
            api_version=self._config.api_version,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    await response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if _stream:
            deserialized = response.iter_bytes()
        else:
            deserialized = _deserialize(_models.PolicyResponse, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore


class PolicyCertificatesOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.security.attestation.aio.AttestationClient`'s
        :attr:`policy_certificates` attribute.
    """

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: AsyncPipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: AttestationClientConfiguration = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace_async
    async def get(self, **kwargs: Any) -> _models.PolicyCertificatesResponse:
        """Retrieves the set of certificates used to express policy for the current tenant.

        Retrieves the set of certificates used to express policy for the current tenant.

        :return: PolicyCertificatesResponse. The PolicyCertificatesResponse is compatible with
         MutableMapping
        :rtype: ~azure.security.attestation.models.PolicyCertificatesResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[_models.PolicyCertificatesResponse] = kwargs.pop("cls", None)

        _request = build_policy_certificates_get_request(
            api_version=self._config.api_version,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    await response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if _stream:
            deserialized = response.iter_bytes()
        else:
            deserialized = _deserialize(_models.PolicyCertificatesResponse, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    async def add(
        self, *, policy_certificate_to_add: str, content_type: str = "application/json", **kwargs: Any
    ) -> _models.PolicyCertificatesModifyResponse:
        """Adds a new attestation policy certificate to the set of policy management
        certificates.

        Adds a new attestation policy certificate to the set of policy management
        certificates.

        :keyword policy_certificate_to_add: The certificate to add, as a string (e.g., PEM or JWK).
         Required.
        :paramtype policy_certificate_to_add: str
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: PolicyCertificatesModifyResponse. The PolicyCertificatesModifyResponse is compatible
         with MutableMapping
        :rtype: ~azure.security.attestation.models.PolicyCertificatesModifyResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def add(
        self, body: JSON, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.PolicyCertificatesModifyResponse:
        """Adds a new attestation policy certificate to the set of policy management
        certificates.

        Adds a new attestation policy certificate to the set of policy management
        certificates.

        :param body: Required.
        :type body: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: PolicyCertificatesModifyResponse. The PolicyCertificatesModifyResponse is compatible
         with MutableMapping
        :rtype: ~azure.security.attestation.models.PolicyCertificatesModifyResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def add(
        self, body: IO[bytes], *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.PolicyCertificatesModifyResponse:
        """Adds a new attestation policy certificate to the set of policy management
        certificates.

        Adds a new attestation policy certificate to the set of policy management
        certificates.

        :param body: Required.
        :type body: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: PolicyCertificatesModifyResponse. The PolicyCertificatesModifyResponse is compatible
         with MutableMapping
        :rtype: ~azure.security.attestation.models.PolicyCertificatesModifyResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def add(
        self, body: Union[JSON, IO[bytes]] = _Unset, *, policy_certificate_to_add: str = _Unset, **kwargs: Any
    ) -> _models.PolicyCertificatesModifyResponse:
        """Adds a new attestation policy certificate to the set of policy management
        certificates.

        Adds a new attestation policy certificate to the set of policy management
        certificates.

        :param body: Is either a JSON type or a IO[bytes] type. Required.
        :type body: JSON or IO[bytes]
        :keyword policy_certificate_to_add: The certificate to add, as a string (e.g., PEM or JWK).
         Required.
        :paramtype policy_certificate_to_add: str
        :return: PolicyCertificatesModifyResponse. The PolicyCertificatesModifyResponse is compatible
         with MutableMapping
        :rtype: ~azure.security.attestation.models.PolicyCertificatesModifyResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.PolicyCertificatesModifyResponse] = kwargs.pop("cls", None)

        if body is _Unset:
            if policy_certificate_to_add is _Unset:
                raise TypeError("missing required argument: policy_certificate_to_add")
            body = {"policyCertificateToAdd": policy_certificate_to_add}
            body = {k: v for k, v in body.items() if v is not None}
        content_type = content_type or "application/json"
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_policy_certificates_add_request(
            content_type=content_type,
            api_version=self._config.api_version,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    await response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if _stream:
            deserialized = response.iter_bytes()
        else:
            deserialized = _deserialize(_models.PolicyCertificatesModifyResponse, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    async def remove(
        self, *, policy_certificate_to_remove: str, content_type: str = "application/json", **kwargs: Any
    ) -> _models.PolicyCertificatesModifyResponse:
        """Removes the specified policy management certificate. Note that the final policy
        management certificate cannot be removed.

        Removes the specified policy management certificate. Note that the final policy
        management certificate cannot be removed.

        :keyword policy_certificate_to_remove: The certificate to remove, as a string (e.g., PEM or
         JWK). Required.
        :paramtype policy_certificate_to_remove: str
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: PolicyCertificatesModifyResponse. The PolicyCertificatesModifyResponse is compatible
         with MutableMapping
        :rtype: ~azure.security.attestation.models.PolicyCertificatesModifyResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def remove(
        self, body: JSON, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.PolicyCertificatesModifyResponse:
        """Removes the specified policy management certificate. Note that the final policy
        management certificate cannot be removed.

        Removes the specified policy management certificate. Note that the final policy
        management certificate cannot be removed.

        :param body: Required.
        :type body: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: PolicyCertificatesModifyResponse. The PolicyCertificatesModifyResponse is compatible
         with MutableMapping
        :rtype: ~azure.security.attestation.models.PolicyCertificatesModifyResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def remove(
        self, body: IO[bytes], *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.PolicyCertificatesModifyResponse:
        """Removes the specified policy management certificate. Note that the final policy
        management certificate cannot be removed.

        Removes the specified policy management certificate. Note that the final policy
        management certificate cannot be removed.

        :param body: Required.
        :type body: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: PolicyCertificatesModifyResponse. The PolicyCertificatesModifyResponse is compatible
         with MutableMapping
        :rtype: ~azure.security.attestation.models.PolicyCertificatesModifyResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def remove(
        self, body: Union[JSON, IO[bytes]] = _Unset, *, policy_certificate_to_remove: str = _Unset, **kwargs: Any
    ) -> _models.PolicyCertificatesModifyResponse:
        """Removes the specified policy management certificate. Note that the final policy
        management certificate cannot be removed.

        Removes the specified policy management certificate. Note that the final policy
        management certificate cannot be removed.

        :param body: Is either a JSON type or a IO[bytes] type. Required.
        :type body: JSON or IO[bytes]
        :keyword policy_certificate_to_remove: The certificate to remove, as a string (e.g., PEM or
         JWK). Required.
        :paramtype policy_certificate_to_remove: str
        :return: PolicyCertificatesModifyResponse. The PolicyCertificatesModifyResponse is compatible
         with MutableMapping
        :rtype: ~azure.security.attestation.models.PolicyCertificatesModifyResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.PolicyCertificatesModifyResponse] = kwargs.pop("cls", None)

        if body is _Unset:
            if policy_certificate_to_remove is _Unset:
                raise TypeError("missing required argument: policy_certificate_to_remove")
            body = {"policyCertificateToRemove": policy_certificate_to_remove}
            body = {k: v for k, v in body.items() if v is not None}
        content_type = content_type or "application/json"
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_policy_certificates_remove_request(
            content_type=content_type,
            api_version=self._config.api_version,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    await response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if _stream:
            deserialized = response.iter_bytes()
        else:
            deserialized = _deserialize(_models.PolicyCertificatesModifyResponse, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore


class AttestationOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.security.attestation.aio.AttestationClient`'s
        :attr:`attestation` attribute.
    """

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: AsyncPipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: AttestationClientConfiguration = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @overload
    async def attest_open_enclave(
        self,
        *,
        content_type: str = "application/json",
        report: Optional[bytes] = None,
        runtime_data: Optional[_models.RuntimeData] = None,
        init_time_data: Optional[_models.InitTimeData] = None,
        draft_policy_for_attestation: Optional[str] = None,
        nonce: Optional[str] = None,
        **kwargs: Any
    ) -> _models.AttestationResponse:
        """Attest to an SGX enclave.

        Processes an OpenEnclave report , producing an artifact. The type of artifact
        produced is dependent upon attestation policy.

        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword report: OpenEnclave report from the enclave to be attested. Default value is None.
        :paramtype report: bytes
        :keyword runtime_data: Runtime data provided by the enclave at the time of report generation.
         The MAA
         will verify that the first 32 bytes of the report_data field of the quote
         contains the SHA256 hash of the decoded "data" field of the runtime data. Default value is
         None.
        :paramtype runtime_data: ~azure.security.attestation.models.RuntimeData
        :keyword init_time_data: Base64Url encoded "InitTime data". The MAA will verify that the init
         data was
         known to the enclave. Note that InitTimeData is invalid for CoffeeLake
         processors. Default value is None.
        :paramtype init_time_data: ~azure.security.attestation.models.InitTimeData
        :keyword draft_policy_for_attestation: Attest against the provided draft policy. Note that the
         resulting token cannot
         be validated. Default value is None.
        :paramtype draft_policy_for_attestation: str
        :keyword nonce: Nonce for incoming request - emitted in the generated attestation token.
         Default value is None.
        :paramtype nonce: str
        :return: AttestationResponse. The AttestationResponse is compatible with MutableMapping
        :rtype: ~azure.security.attestation.models.AttestationResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def attest_open_enclave(
        self, body: JSON, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.AttestationResponse:
        """Attest to an SGX enclave.

        Processes an OpenEnclave report , producing an artifact. The type of artifact
        produced is dependent upon attestation policy.

        :param body: Required.
        :type body: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: AttestationResponse. The AttestationResponse is compatible with MutableMapping
        :rtype: ~azure.security.attestation.models.AttestationResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def attest_open_enclave(
        self, body: IO[bytes], *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.AttestationResponse:
        """Attest to an SGX enclave.

        Processes an OpenEnclave report , producing an artifact. The type of artifact
        produced is dependent upon attestation policy.

        :param body: Required.
        :type body: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: AttestationResponse. The AttestationResponse is compatible with MutableMapping
        :rtype: ~azure.security.attestation.models.AttestationResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def attest_open_enclave(
        self,
        body: Union[JSON, IO[bytes]] = _Unset,
        *,
        report: Optional[bytes] = None,
        runtime_data: Optional[_models.RuntimeData] = None,
        init_time_data: Optional[_models.InitTimeData] = None,
        draft_policy_for_attestation: Optional[str] = None,
        nonce: Optional[str] = None,
        **kwargs: Any
    ) -> _models.AttestationResponse:
        """Attest to an SGX enclave.

        Processes an OpenEnclave report , producing an artifact. The type of artifact
        produced is dependent upon attestation policy.

        :param body: Is either a JSON type or a IO[bytes] type. Required.
        :type body: JSON or IO[bytes]
        :keyword report: OpenEnclave report from the enclave to be attested. Default value is None.
        :paramtype report: bytes
        :keyword runtime_data: Runtime data provided by the enclave at the time of report generation.
         The MAA
         will verify that the first 32 bytes of the report_data field of the quote
         contains the SHA256 hash of the decoded "data" field of the runtime data. Default value is
         None.
        :paramtype runtime_data: ~azure.security.attestation.models.RuntimeData
        :keyword init_time_data: Base64Url encoded "InitTime data". The MAA will verify that the init
         data was
         known to the enclave. Note that InitTimeData is invalid for CoffeeLake
         processors. Default value is None.
        :paramtype init_time_data: ~azure.security.attestation.models.InitTimeData
        :keyword draft_policy_for_attestation: Attest against the provided draft policy. Note that the
         resulting token cannot
         be validated. Default value is None.
        :paramtype draft_policy_for_attestation: str
        :keyword nonce: Nonce for incoming request - emitted in the generated attestation token.
         Default value is None.
        :paramtype nonce: str
        :return: AttestationResponse. The AttestationResponse is compatible with MutableMapping
        :rtype: ~azure.security.attestation.models.AttestationResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.AttestationResponse] = kwargs.pop("cls", None)

        if body is _Unset:
            body = {
                "draftPolicyForAttestation": draft_policy_for_attestation,
                "initTimeData": init_time_data,
                "nonce": nonce,
                "report": report,
                "runtimeData": runtime_data,
            }
            body = {k: v for k, v in body.items() if v is not None}
        content_type = content_type or "application/json"
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_attestation_attest_open_enclave_request(
            content_type=content_type,
            api_version=self._config.api_version,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    await response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if _stream:
            deserialized = response.iter_bytes()
        else:
            deserialized = _deserialize(_models.AttestationResponse, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    async def attest_sgx_enclave(
        self,
        *,
        content_type: str = "application/json",
        quote: Optional[bytes] = None,
        runtime_data: Optional[_models.RuntimeData] = None,
        init_time_data: Optional[_models.InitTimeData] = None,
        draft_policy_for_attestation: Optional[str] = None,
        nonce: Optional[str] = None,
        **kwargs: Any
    ) -> _models.AttestationResponse:
        """Attest to an SGX enclave.

        Processes an SGX enclave quote, producing an artifact. The type of artifact
        produced is dependent upon attestation policy.

        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword quote: Quote of the enclave to be attested. Default value is None.
        :paramtype quote: bytes
        :keyword runtime_data: Runtime data provided by the enclave at the time of quote generation.
         The MAA
         will verify that the first 32 bytes of the report_data field of the quote
         contains the SHA256 hash of the decoded "data" field of the runtime data. Default value is
         None.
        :paramtype runtime_data: ~azure.security.attestation.models.RuntimeData
        :keyword init_time_data: Initialization data provided when the enclave is created. MAA will
         verify that
         the init data was known to the enclave. Note that InitTimeData is invalid for
         CoffeeLake processors. Default value is None.
        :paramtype init_time_data: ~azure.security.attestation.models.InitTimeData
        :keyword draft_policy_for_attestation: Attest against the provided draft policy. Note that the
         resulting token cannot
         be validated. Default value is None.
        :paramtype draft_policy_for_attestation: str
        :keyword nonce: Nonce for incoming request - emitted in the generated attestation token.
         Default value is None.
        :paramtype nonce: str
        :return: AttestationResponse. The AttestationResponse is compatible with MutableMapping
        :rtype: ~azure.security.attestation.models.AttestationResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def attest_sgx_enclave(
        self, body: JSON, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.AttestationResponse:
        """Attest to an SGX enclave.

        Processes an SGX enclave quote, producing an artifact. The type of artifact
        produced is dependent upon attestation policy.

        :param body: Required.
        :type body: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: AttestationResponse. The AttestationResponse is compatible with MutableMapping
        :rtype: ~azure.security.attestation.models.AttestationResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def attest_sgx_enclave(
        self, body: IO[bytes], *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.AttestationResponse:
        """Attest to an SGX enclave.

        Processes an SGX enclave quote, producing an artifact. The type of artifact
        produced is dependent upon attestation policy.

        :param body: Required.
        :type body: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: AttestationResponse. The AttestationResponse is compatible with MutableMapping
        :rtype: ~azure.security.attestation.models.AttestationResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def attest_sgx_enclave(
        self,
        body: Union[JSON, IO[bytes]] = _Unset,
        *,
        quote: Optional[bytes] = None,
        runtime_data: Optional[_models.RuntimeData] = None,
        init_time_data: Optional[_models.InitTimeData] = None,
        draft_policy_for_attestation: Optional[str] = None,
        nonce: Optional[str] = None,
        **kwargs: Any
    ) -> _models.AttestationResponse:
        """Attest to an SGX enclave.

        Processes an SGX enclave quote, producing an artifact. The type of artifact
        produced is dependent upon attestation policy.

        :param body: Is either a JSON type or a IO[bytes] type. Required.
        :type body: JSON or IO[bytes]
        :keyword quote: Quote of the enclave to be attested. Default value is None.
        :paramtype quote: bytes
        :keyword runtime_data: Runtime data provided by the enclave at the time of quote generation.
         The MAA
         will verify that the first 32 bytes of the report_data field of the quote
         contains the SHA256 hash of the decoded "data" field of the runtime data. Default value is
         None.
        :paramtype runtime_data: ~azure.security.attestation.models.RuntimeData
        :keyword init_time_data: Initialization data provided when the enclave is created. MAA will
         verify that
         the init data was known to the enclave. Note that InitTimeData is invalid for
         CoffeeLake processors. Default value is None.
        :paramtype init_time_data: ~azure.security.attestation.models.InitTimeData
        :keyword draft_policy_for_attestation: Attest against the provided draft policy. Note that the
         resulting token cannot
         be validated. Default value is None.
        :paramtype draft_policy_for_attestation: str
        :keyword nonce: Nonce for incoming request - emitted in the generated attestation token.
         Default value is None.
        :paramtype nonce: str
        :return: AttestationResponse. The AttestationResponse is compatible with MutableMapping
        :rtype: ~azure.security.attestation.models.AttestationResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.AttestationResponse] = kwargs.pop("cls", None)

        if body is _Unset:
            body = {
                "draftPolicyForAttestation": draft_policy_for_attestation,
                "initTimeData": init_time_data,
                "nonce": nonce,
                "quote": quote,
                "runtimeData": runtime_data,
            }
            body = {k: v for k, v in body.items() if v is not None}
        content_type = content_type or "application/json"
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_attestation_attest_sgx_enclave_request(
            content_type=content_type,
            api_version=self._config.api_version,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    await response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if _stream:
            deserialized = response.iter_bytes()
        else:
            deserialized = _deserialize(_models.AttestationResponse, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    async def attest_azure_guest(
        self, *, content_type: str = "application/json", attestation_info: Optional[str] = None, **kwargs: Any
    ) -> _models.SealedAttestationResponse:
        """Attest to an Azure Guest VM.

        Processes an Azure Guest TCG Log, producing an artifact. The type of artifact
        produced is dependent upon attestation policy.

        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword attestation_info: Attestation client information containing all artifacts required for
         Guest
         Attestation. Default value is None.
        :paramtype attestation_info: str
        :return: SealedAttestationResponse. The SealedAttestationResponse is compatible with
         MutableMapping
        :rtype: ~azure.security.attestation.models.SealedAttestationResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def attest_azure_guest(
        self, body: JSON, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.SealedAttestationResponse:
        """Attest to an Azure Guest VM.

        Processes an Azure Guest TCG Log, producing an artifact. The type of artifact
        produced is dependent upon attestation policy.

        :param body: Required.
        :type body: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: SealedAttestationResponse. The SealedAttestationResponse is compatible with
         MutableMapping
        :rtype: ~azure.security.attestation.models.SealedAttestationResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def attest_azure_guest(
        self, body: IO[bytes], *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.SealedAttestationResponse:
        """Attest to an Azure Guest VM.

        Processes an Azure Guest TCG Log, producing an artifact. The type of artifact
        produced is dependent upon attestation policy.

        :param body: Required.
        :type body: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: SealedAttestationResponse. The SealedAttestationResponse is compatible with
         MutableMapping
        :rtype: ~azure.security.attestation.models.SealedAttestationResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def attest_azure_guest(
        self, body: Union[JSON, IO[bytes]] = _Unset, *, attestation_info: Optional[str] = None, **kwargs: Any
    ) -> _models.SealedAttestationResponse:
        """Attest to an Azure Guest VM.

        Processes an Azure Guest TCG Log, producing an artifact. The type of artifact
        produced is dependent upon attestation policy.

        :param body: Is either a JSON type or a IO[bytes] type. Required.
        :type body: JSON or IO[bytes]
        :keyword attestation_info: Attestation client information containing all artifacts required for
         Guest
         Attestation. Default value is None.
        :paramtype attestation_info: str
        :return: SealedAttestationResponse. The SealedAttestationResponse is compatible with
         MutableMapping
        :rtype: ~azure.security.attestation.models.SealedAttestationResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.SealedAttestationResponse] = kwargs.pop("cls", None)

        if body is _Unset:
            body = {"attestationInfo": attestation_info}
            body = {k: v for k, v in body.items() if v is not None}
        content_type = content_type or "application/json"
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_attestation_attest_azure_guest_request(
            content_type=content_type,
            api_version=self._config.api_version,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    await response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if _stream:
            deserialized = response.iter_bytes()
        else:
            deserialized = _deserialize(_models.SealedAttestationResponse, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    async def attest_tpm(
        self, *, content_type: str = "application/json", data: Optional[bytes] = None, **kwargs: Any
    ) -> _models.TpmAttestationResponse:
        """Attest a Virtualization-based Security (VBS) enclave.

        Processes attestation evidence from a VBS enclave, producing an attestation
        result. The attestation result produced is dependent upon the attestation
        policy.

        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword data: Protocol data containing artifacts for attestation. Default value is None.
        :paramtype data: bytes
        :return: TpmAttestationResponse. The TpmAttestationResponse is compatible with MutableMapping
        :rtype: ~azure.security.attestation.models.TpmAttestationResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def attest_tpm(
        self, body: JSON, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.TpmAttestationResponse:
        """Attest a Virtualization-based Security (VBS) enclave.

        Processes attestation evidence from a VBS enclave, producing an attestation
        result. The attestation result produced is dependent upon the attestation
        policy.

        :param body: Required.
        :type body: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: TpmAttestationResponse. The TpmAttestationResponse is compatible with MutableMapping
        :rtype: ~azure.security.attestation.models.TpmAttestationResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def attest_tpm(
        self, body: IO[bytes], *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.TpmAttestationResponse:
        """Attest a Virtualization-based Security (VBS) enclave.

        Processes attestation evidence from a VBS enclave, producing an attestation
        result. The attestation result produced is dependent upon the attestation
        policy.

        :param body: Required.
        :type body: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: TpmAttestationResponse. The TpmAttestationResponse is compatible with MutableMapping
        :rtype: ~azure.security.attestation.models.TpmAttestationResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def attest_tpm(
        self, body: Union[JSON, IO[bytes]] = _Unset, *, data: Optional[bytes] = None, **kwargs: Any
    ) -> _models.TpmAttestationResponse:
        """Attest a Virtualization-based Security (VBS) enclave.

        Processes attestation evidence from a VBS enclave, producing an attestation
        result. The attestation result produced is dependent upon the attestation
        policy.

        :param body: Is either a JSON type or a IO[bytes] type. Required.
        :type body: JSON or IO[bytes]
        :keyword data: Protocol data containing artifacts for attestation. Default value is None.
        :paramtype data: bytes
        :return: TpmAttestationResponse. The TpmAttestationResponse is compatible with MutableMapping
        :rtype: ~azure.security.attestation.models.TpmAttestationResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.TpmAttestationResponse] = kwargs.pop("cls", None)

        if body is _Unset:
            body = {"data": data}
            body = {k: v for k, v in body.items() if v is not None}
        content_type = content_type or "application/json"
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_attestation_attest_tpm_request(
            content_type=content_type,
            api_version=self._config.api_version,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    await response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if _stream:
            deserialized = response.iter_bytes()
        else:
            deserialized = _deserialize(_models.TpmAttestationResponse, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    async def attest_sev_snp_vm(
        self,
        *,
        content_type: str = "application/json",
        report: Optional[str] = None,
        runtime_data: Optional[_models.RuntimeData] = None,
        init_time_data: Optional[_models.InitTimeData] = None,
        draft_policy_for_attestation: Optional[str] = None,
        nonce: Optional[str] = None,
        **kwargs: Any
    ) -> _models.AttestationResponse:
        """Attest to an SEV SNP Virtual Machine.

        Processes a SEV SNP Boot chain. The type of artifact produced is dependent upon
        attestation policy.

        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword report: Hardware rooted report of the virtual machine being attested along with the
         signing certificate chain and optionally, additional endorsements. Default value is None.
        :paramtype report: str
        :keyword runtime_data: Runtime data provided by the enclave at the time of report generation.
         The MAA
         will verify that the run time data is known to the attestation target. Default value is None.
        :paramtype runtime_data: ~azure.security.attestation.models.RuntimeData
        :keyword init_time_data: Initialization data provided by the enclave at the time of report
         generation.
         The MAA will verify that the init time data is known to the attestation target. Default value
         is None.
        :paramtype init_time_data: ~azure.security.attestation.models.InitTimeData
        :keyword draft_policy_for_attestation: Attest against the provided draft policy. Note that the
         resulting token cannot
         be validated. Default value is None.
        :paramtype draft_policy_for_attestation: str
        :keyword nonce: Nonce for incoming request - emitted in the generated attestation token.
         Default value is None.
        :paramtype nonce: str
        :return: AttestationResponse. The AttestationResponse is compatible with MutableMapping
        :rtype: ~azure.security.attestation.models.AttestationResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def attest_sev_snp_vm(
        self, body: JSON, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.AttestationResponse:
        """Attest to an SEV SNP Virtual Machine.

        Processes a SEV SNP Boot chain. The type of artifact produced is dependent upon
        attestation policy.

        :param body: Required.
        :type body: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: AttestationResponse. The AttestationResponse is compatible with MutableMapping
        :rtype: ~azure.security.attestation.models.AttestationResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def attest_sev_snp_vm(
        self, body: IO[bytes], *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.AttestationResponse:
        """Attest to an SEV SNP Virtual Machine.

        Processes a SEV SNP Boot chain. The type of artifact produced is dependent upon
        attestation policy.

        :param body: Required.
        :type body: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: AttestationResponse. The AttestationResponse is compatible with MutableMapping
        :rtype: ~azure.security.attestation.models.AttestationResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def attest_sev_snp_vm(
        self,
        body: Union[JSON, IO[bytes]] = _Unset,
        *,
        report: Optional[str] = None,
        runtime_data: Optional[_models.RuntimeData] = None,
        init_time_data: Optional[_models.InitTimeData] = None,
        draft_policy_for_attestation: Optional[str] = None,
        nonce: Optional[str] = None,
        **kwargs: Any
    ) -> _models.AttestationResponse:
        """Attest to an SEV SNP Virtual Machine.

        Processes a SEV SNP Boot chain. The type of artifact produced is dependent upon
        attestation policy.

        :param body: Is either a JSON type or a IO[bytes] type. Required.
        :type body: JSON or IO[bytes]
        :keyword report: Hardware rooted report of the virtual machine being attested along with the
         signing certificate chain and optionally, additional endorsements. Default value is None.
        :paramtype report: str
        :keyword runtime_data: Runtime data provided by the enclave at the time of report generation.
         The MAA
         will verify that the run time data is known to the attestation target. Default value is None.
        :paramtype runtime_data: ~azure.security.attestation.models.RuntimeData
        :keyword init_time_data: Initialization data provided by the enclave at the time of report
         generation.
         The MAA will verify that the init time data is known to the attestation target. Default value
         is None.
        :paramtype init_time_data: ~azure.security.attestation.models.InitTimeData
        :keyword draft_policy_for_attestation: Attest against the provided draft policy. Note that the
         resulting token cannot
         be validated. Default value is None.
        :paramtype draft_policy_for_attestation: str
        :keyword nonce: Nonce for incoming request - emitted in the generated attestation token.
         Default value is None.
        :paramtype nonce: str
        :return: AttestationResponse. The AttestationResponse is compatible with MutableMapping
        :rtype: ~azure.security.attestation.models.AttestationResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.AttestationResponse] = kwargs.pop("cls", None)

        if body is _Unset:
            body = {
                "draftPolicyForAttestation": draft_policy_for_attestation,
                "initTimeData": init_time_data,
                "nonce": nonce,
                "report": report,
                "runtimeData": runtime_data,
            }
            body = {k: v for k, v in body.items() if v is not None}
        content_type = content_type or "application/json"
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_attestation_attest_sev_snp_vm_request(
            content_type=content_type,
            api_version=self._config.api_version,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    await response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if _stream:
            deserialized = response.iter_bytes()
        else:
            deserialized = _deserialize(_models.AttestationResponse, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    async def attest_tdx_vm(
        self,
        *,
        content_type: str = "application/json",
        quote: Optional[bytes] = None,
        runtime_data: Optional[_models.RuntimeData] = None,
        init_time_data: Optional[_models.InitTimeData] = None,
        nonce: Optional[str] = None,
        **kwargs: Any
    ) -> _models.AttestationResponse:
        """Attest to a TDX Virtual Machine.

        Processes an TDX quote, producing an artifact. The type of artifact produced is
        dependent upon attestation policy.

        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword quote: Quote of the TDX virtual machine to be attested. Default value is None.
        :paramtype quote: bytes
        :keyword runtime_data: Runtime data provided by the enclave at the time of quote generation.
         The MAA
         will verify that the first 32 bytes of the report_data field of the quote
         contains the SHA256 hash of the decoded "data" field of the runtime data. Default value is
         None.
        :paramtype runtime_data: ~azure.security.attestation.models.RuntimeData
        :keyword init_time_data: Initialization data provided when the enclave is created. MAA will
         verify that
         the init data was known to the enclave. Default value is None.
        :paramtype init_time_data: ~azure.security.attestation.models.InitTimeData
        :keyword nonce: Nonce for incoming request - emitted in the generated attestation token.
         Default value is None.
        :paramtype nonce: str
        :return: AttestationResponse. The AttestationResponse is compatible with MutableMapping
        :rtype: ~azure.security.attestation.models.AttestationResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def attest_tdx_vm(
        self, body: JSON, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.AttestationResponse:
        """Attest to a TDX Virtual Machine.

        Processes an TDX quote, producing an artifact. The type of artifact produced is
        dependent upon attestation policy.

        :param body: Required.
        :type body: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: AttestationResponse. The AttestationResponse is compatible with MutableMapping
        :rtype: ~azure.security.attestation.models.AttestationResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def attest_tdx_vm(
        self, body: IO[bytes], *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.AttestationResponse:
        """Attest to a TDX Virtual Machine.

        Processes an TDX quote, producing an artifact. The type of artifact produced is
        dependent upon attestation policy.

        :param body: Required.
        :type body: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: AttestationResponse. The AttestationResponse is compatible with MutableMapping
        :rtype: ~azure.security.attestation.models.AttestationResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def attest_tdx_vm(
        self,
        body: Union[JSON, IO[bytes]] = _Unset,
        *,
        quote: Optional[bytes] = None,
        runtime_data: Optional[_models.RuntimeData] = None,
        init_time_data: Optional[_models.InitTimeData] = None,
        nonce: Optional[str] = None,
        **kwargs: Any
    ) -> _models.AttestationResponse:
        """Attest to a TDX Virtual Machine.

        Processes an TDX quote, producing an artifact. The type of artifact produced is
        dependent upon attestation policy.

        :param body: Is either a JSON type or a IO[bytes] type. Required.
        :type body: JSON or IO[bytes]
        :keyword quote: Quote of the TDX virtual machine to be attested. Default value is None.
        :paramtype quote: bytes
        :keyword runtime_data: Runtime data provided by the enclave at the time of quote generation.
         The MAA
         will verify that the first 32 bytes of the report_data field of the quote
         contains the SHA256 hash of the decoded "data" field of the runtime data. Default value is
         None.
        :paramtype runtime_data: ~azure.security.attestation.models.RuntimeData
        :keyword init_time_data: Initialization data provided when the enclave is created. MAA will
         verify that
         the init data was known to the enclave. Default value is None.
        :paramtype init_time_data: ~azure.security.attestation.models.InitTimeData
        :keyword nonce: Nonce for incoming request - emitted in the generated attestation token.
         Default value is None.
        :paramtype nonce: str
        :return: AttestationResponse. The AttestationResponse is compatible with MutableMapping
        :rtype: ~azure.security.attestation.models.AttestationResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.AttestationResponse] = kwargs.pop("cls", None)

        if body is _Unset:
            body = {"initTimeData": init_time_data, "nonce": nonce, "quote": quote, "runtimeData": runtime_data}
            body = {k: v for k, v in body.items() if v is not None}
        content_type = content_type or "application/json"
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_attestation_attest_tdx_vm_request(
            content_type=content_type,
            api_version=self._config.api_version,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    await response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if _stream:
            deserialized = response.iter_bytes()
        else:
            deserialized = _deserialize(_models.AttestationResponse, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore


class TcbBaselinesOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.security.attestation.aio.AttestationClient`'s
        :attr:`tcb_baselines` attribute.
    """

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: AsyncPipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: AttestationClientConfiguration = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace_async
    async def get(
        self, attestation_type: Union[str, _models.AttestationType], **kwargs: Any
    ) -> _models.TcbBaselineResult:
        """Retrieves a list of all Azure supported baseline details for the a TEE type
        along with an indication of which one is in enforced for the attestation
        provider.

        Retrieves a list of all Azure supported baseline details for the a TEE type
        along with an indication of which one is in enforced for the attestation
        provider.

        :param attestation_type: Specifies the trusted execution environment to be used to validate the
         evidence. Known values are: "SgxEnclave", "OpenEnclave", "AzureGuest", "SevSnpVm", "Tpm", and
         "TdxVm". Required.
        :type attestation_type: str or ~azure.security.attestation.models.AttestationType
        :return: TcbBaselineResult. The TcbBaselineResult is compatible with MutableMapping
        :rtype: ~azure.security.attestation.models.TcbBaselineResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[_models.TcbBaselineResult] = kwargs.pop("cls", None)

        _request = build_tcb_baselines_get_request(
            attestation_type=attestation_type,
            api_version=self._config.api_version,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    await response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if _stream:
            deserialized = response.iter_bytes()
        else:
            deserialized = _deserialize(_models.TcbBaselineResult, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore


class SigningCertificatesOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.security.attestation.aio.AttestationClient`'s
        :attr:`signing_certificates` attribute.
    """

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: AsyncPipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: AttestationClientConfiguration = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace_async
    async def get(self, **kwargs: Any) -> _models.JsonWebKeySet:
        """Retrieves the attestation signing keys in use by the attestation service.

        Retrieves metadata signing certificates in use by the attestation service.

        :return: JsonWebKeySet. The JsonWebKeySet is compatible with MutableMapping
        :rtype: ~azure.security.attestation.models.JsonWebKeySet
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[_models.JsonWebKeySet] = kwargs.pop("cls", None)

        _request = build_signing_certificates_get_request(
            api_version=self._config.api_version,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    await response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        response_headers = {}
        response_headers["content-type"] = self._deserialize("str", response.headers.get("content-type"))

        if _stream:
            deserialized = response.iter_bytes()
        else:
            deserialized = _deserialize(_models.JsonWebKeySet, response.json())

        if cls:
            return cls(pipeline_response, deserialized, response_headers)  # type: ignore

        return deserialized  # type: ignore


class MetadataConfigurationOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.security.attestation.aio.AttestationClient`'s
        :attr:`metadata_configuration` attribute.
    """

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: AsyncPipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: AttestationClientConfiguration = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace_async
    async def get(self, **kwargs: Any) -> _models.OpenIDConfigurationResponse:
        """Retrieves the OpenID Configuration data for the Azure Attestation Service.

        Retrieves metadata about the attestation signing keys in use by the attestation
        service.

        :return: OpenIDConfigurationResponse. The OpenIDConfigurationResponse is compatible with
         MutableMapping
        :rtype: ~azure.security.attestation.models.OpenIDConfigurationResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[_models.OpenIDConfigurationResponse] = kwargs.pop("cls", None)

        _request = build_metadata_configuration_get_request(
            api_version=self._config.api_version,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    await response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if _stream:
            deserialized = response.iter_bytes()
        else:
            deserialized = _deserialize(_models.OpenIDConfigurationResponse, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore



# coding=utf-8
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import


from ._models import (  # type: ignore
    Author,
    Book,
    ModelWithArrayOfModel,
    ModelWithAttributes,
    ModelWithDatetime,
    ModelWithDictionary,
    ModelWithEmptyArray,
    ModelWithEncodedNames,
    ModelWithEnum,
    ModelWithNamespace,
    ModelWithNamespaceOnProperties,
    ModelWithNestedModel,
    ModelWithOptionalField,
    ModelWithRenamedArrays,
    ModelWithRenamedAttribute,
    ModelWithRenamedFields,
    ModelWithRenamedNestedModel,
    ModelWithRenamedProperty,
    ModelWithRenamedUnwrappedModelArray,
    ModelWithRenamedWrappedAndItemModelArray,
    ModelWithRenamedWrappedModelArray,
    ModelWithSimpleArrays,
    ModelWithText,
    ModelWithUnwrappedArray,
    ModelWithUnwrappedModelArray,
    ModelWithWrappedPrimitiveCustomItemNames,
    SimpleModel,
    XmlErrorBody,
)

from ._enums import (  # type: ignore
    Status,
)
from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "Author",
    "Book",
    "ModelWithArrayOfModel",
    "ModelWithAttributes",
    "ModelWithDatetime",
    "ModelWithDictionary",
    "ModelWithEmptyArray",
    "ModelWithEncodedNames",
    "ModelWithEnum",
    "ModelWithNamespace",
    "ModelWithNamespaceOnProperties",
    "ModelWithNestedModel",
    "ModelWithOptionalField",
    "ModelWithRenamedArrays",
    "ModelWithRenamedAttribute",
    "ModelWithRenamedFields",
    "ModelWithRenamedNestedModel",
    "ModelWithRenamedProperty",
    "ModelWithRenamedUnwrappedModelArray",
    "ModelWithRenamedWrappedAndItemModelArray",
    "ModelWithRenamedWrappedModelArray",
    "ModelWithSimpleArrays",
    "ModelWithText",
    "ModelWithUnwrappedArray",
    "ModelWithUnwrappedModelArray",
    "ModelWithWrappedPrimitiveCustomItemNames",
    "SimpleModel",
    "XmlErrorBody",
    "Status",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
from ._communication_identity_client import CommunicationIdentityClient
from ._generated.models import CommunicationTokenScope

from ._shared.models import (
    CommunicationCloudEnvironment,
    CommunicationIdentifier,
    CommunicationIdentifierKind,
    CommunicationUserIdentifier,
    CommunicationUserProperties,
    identifier_from_raw_id,
    MicrosoftTeamsAppIdentifier,
    MicrosoftTeamsAppProperties,
    MicrosoftTeamsUserIdentifier,
    MicrosoftTeamsUserProperties,
    PhoneNumberIdentifier,
    PhoneNumberProperties,
    UnknownIdentifier,
    TeamsExtensionUserIdentifier,
    TeamsExtensionUserProperties,
)

__all__ = [
    "CommunicationIdentityClient",
    # from _identity
    "CommunicationTokenScope",
    # from _shared
    "CommunicationCloudEnvironment",
    "CommunicationIdentifier",
    "CommunicationIdentifierKind",
    "CommunicationUserIdentifier",
    "CommunicationUserProperties",
    "identifier_from_raw_id",
    "MicrosoftTeamsAppIdentifier",
    "MicrosoftTeamsAppProperties",
    "MicrosoftTeamsUserIdentifier",
    "MicrosoftTeamsUserProperties",
    "PhoneNumberIdentifier",
    "PhoneNumberProperties",
    "UnknownIdentifier",
    "TeamsExtensionUserIdentifier",
    "TeamsExtensionUserProperties",
]



# coding=utf-8
from collections.abc import MutableMapping
from typing import Any, Callable, Optional, TypeVar

from corehttp.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from corehttp.rest import HttpRequest, HttpResponse
from corehttp.runtime import PipelineClient
from corehttp.runtime.pipeline import PipelineResponse

from .._configuration import PathClientConfiguration
from .._utils.serialization import Serializer
from .._utils.utils import ClientMixinABC

T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, dict[str, Any]], Any]]

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False


def build_path_normal_request(name: str, **kwargs: Any) -> HttpRequest:
    # Construct URL
    _url = "/parameters/path/normal/{name}"
    path_format_arguments = {
        "name": _SERIALIZER.url("name", name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    return HttpRequest(method="GET", url=_url, **kwargs)


def build_path_optional_request(*, name: Optional[str] = None, **kwargs: Any) -> HttpRequest:
    # Construct URL
    _url = "/parameters/path/optional{name}"
    path_format_arguments = {
        "name": "" if name is None else "/" + _SERIALIZER.url("name", name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    return HttpRequest(method="GET", url=_url, **kwargs)


class _PathClientOperationsMixin(ClientMixinABC[PipelineClient[HttpRequest, HttpResponse], PathClientConfiguration]):

    def normal(self, name: str, **kwargs: Any) -> None:  # pylint: disable=inconsistent-return-statements
        """normal.

        :param name: Required.
        :type name: str
        :return: None
        :rtype: None
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_path_normal_request(
            name=name,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client.pipeline.run(_request, stream=_stream, **kwargs)

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    def optional(  # pylint: disable=inconsistent-return-statements
        self, *, name: Optional[str] = None, **kwargs: Any
    ) -> None:
        """optional.

        :keyword name: Default value is None.
        :paramtype name: str
        :return: None
        :rtype: None
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_path_optional_request(
            name=name,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client.pipeline.run(_request, stream=_stream, **kwargs)

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore



# coding=utf-8
from collections.abc import MutableMapping
import json
from typing import Any, Callable, Optional, TypeVar

from corehttp.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    StreamClosedError,
    StreamConsumedError,
    map_error,
)
from corehttp.rest import HttpRequest, HttpResponse
from corehttp.runtime import PipelineClient
from corehttp.runtime.pipeline import PipelineResponse
from corehttp.utils import case_insensitive_dict

from ..._configuration import MediaTypeClientConfiguration
from ..._utils.model_base import SdkJSONEncoder, _deserialize
from ..._utils.serialization import Deserializer, Serializer

T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, dict[str, Any]], Any]]

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False


def build_string_body_send_as_text_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type: str = kwargs.pop("content_type")
    # Construct URL
    _url = "/payload/media-type/string-body/sendAsText"

    # Construct headers
    _headers["content-type"] = _SERIALIZER.header("content_type", content_type, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_string_body_get_as_text_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "text/plain")

    # Construct URL
    _url = "/payload/media-type/string-body/getAsText"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_string_body_send_as_json_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type: str = kwargs.pop("content_type")
    # Construct URL
    _url = "/payload/media-type/string-body/sendAsJson"

    # Construct headers
    _headers["content-type"] = _SERIALIZER.header("content_type", content_type, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_string_body_get_as_json_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/payload/media-type/string-body/getAsJson"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


class StringBodyOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~payload.mediatype.MediaTypeClient`'s
        :attr:`string_body` attribute.
    """

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: PipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: MediaTypeClientConfiguration = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    def send_as_text(self, text: str, **kwargs: Any) -> None:  # pylint: disable=inconsistent-return-statements
        """send_as_text.

        :param text: Required.
        :type text: str
        :return: None
        :rtype: None
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: str = kwargs.pop("content_type", _headers.pop("content-type", "text/plain"))
        cls: ClsType[None] = kwargs.pop("cls", None)

        _content = text

        _request = build_string_body_send_as_text_request(
            content_type=content_type,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client.pipeline.run(_request, stream=_stream, **kwargs)

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    def get_as_text(self, **kwargs: Any) -> str:
        """get_as_text.

        :return: str
        :rtype: str
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[str] = kwargs.pop("cls", None)

        _request = build_string_body_get_as_text_request(
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _decompress = kwargs.pop("decompress", True)
        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = self._client.pipeline.run(_request, stream=_stream, **kwargs)

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        response_headers = {}
        response_headers["content-type"] = self._deserialize("str", response.headers.get("content-type"))

        if _stream:
            deserialized = response.iter_bytes() if _decompress else response.iter_raw()
        else:
            deserialized = _deserialize(str, response.text())

        if cls:
            return cls(pipeline_response, deserialized, response_headers)  # type: ignore

        return deserialized  # type: ignore

    def send_as_json(self, text: str, **kwargs: Any) -> None:  # pylint: disable=inconsistent-return-statements
        """send_as_json.

        :param text: Required.
        :type text: str
        :return: None
        :rtype: None
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: str = kwargs.pop("content_type", _headers.pop("content-type", "application/json"))
        cls: ClsType[None] = kwargs.pop("cls", None)

        _content = json.dumps(text, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_string_body_send_as_json_request(
            content_type=content_type,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client.pipeline.run(_request, stream=_stream, **kwargs)

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    def get_as_json(self, **kwargs: Any) -> str:
        """get_as_json.

        :return: str
        :rtype: str
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[str] = kwargs.pop("cls", None)

        _request = build_string_body_get_as_json_request(
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _decompress = kwargs.pop("decompress", True)
        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = self._client.pipeline.run(_request, stream=_stream, **kwargs)

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        response_headers = {}
        response_headers["content-type"] = self._deserialize("str", response.headers.get("content-type"))

        if _stream:
            deserialized = response.iter_bytes() if _decompress else response.iter_raw()
        else:
            deserialized = _deserialize(str, response.json())

        if cls:
            return cls(pipeline_response, deserialized, response_headers)  # type: ignore

        return deserialized  # type: ignore



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class ExtendedEnum(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of ExtendedEnum."""

    ENUM_VALUE2 = "value2"
    """ENUM_VALUE2."""


class FixedInnerEnum(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enum that will be used as a property for model EnumProperty. Non-extensible."""

    VALUE_ONE = "ValueOne"
    """First value."""
    VALUE_TWO = "ValueTwo"
    """Second value."""


class InnerEnum(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enum that will be used as a property for model EnumProperty. Extensible."""

    VALUE_ONE = "ValueOne"
    """First value."""
    VALUE_TWO = "ValueTwo"
    """Second value."""



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.billing.aio import BillingManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestBillingManagementProductsOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(BillingManagementClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_list_by_invoice_section(self, resource_group):
        response = self.client.products.list_by_invoice_section(
            billing_account_name="str",
            billing_profile_name="str",
            invoice_section_name="str",
            api_version="2024-04-01",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_list_by_billing_profile(self, resource_group):
        response = self.client.products.list_by_billing_profile(
            billing_account_name="str",
            billing_profile_name="str",
            api_version="2024-04-01",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_list_by_customer(self, resource_group):
        response = self.client.products.list_by_customer(
            billing_account_name="str",
            customer_name="str",
            api_version="2024-04-01",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_begin_move(self, resource_group):
        response = await (
            await self.client.products.begin_move(
                billing_account_name="str",
                product_name="str",
                parameters={"destinationInvoiceSectionId": "str"},
                api_version="2024-04-01",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_validate_move_eligibility(self, resource_group):
        response = await self.client.products.validate_move_eligibility(
            billing_account_name="str",
            product_name="str",
            parameters={"destinationInvoiceSectionId": "str"},
            api_version="2024-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_get(self, resource_group):
        response = await self.client.products.get(
            billing_account_name="str",
            product_name="str",
            api_version="2024-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_update(self, resource_group):
        response = await self.client.products.update(
            billing_account_name="str",
            product_name="str",
            parameters={
                "id": "str",
                "name": "str",
                "properties": {
                    "autoRenew": "str",
                    "availabilityId": "str",
                    "billingFrequency": "str",
                    "billingProfileDisplayName": "str",
                    "billingProfileId": "str",
                    "customerDisplayName": "str",
                    "customerId": "str",
                    "displayName": "str",
                    "endDate": "str",
                    "invoiceSectionDisplayName": "str",
                    "invoiceSectionId": "str",
                    "lastCharge": {"currency": "str", "value": 0.0},
                    "lastChargeDate": "str",
                    "productType": "str",
                    "productTypeId": "str",
                    "purchaseDate": "str",
                    "quantity": 0,
                    "reseller": {"description": "str", "resellerId": "str"},
                    "skuDescription": "str",
                    "skuId": "str",
                    "status": "str",
                    "tenantId": "str",
                },
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "tags": {"str": "str"},
                "type": "str",
            },
            api_version="2024-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_list_by_billing_account(self, resource_group):
        response = self.client.products.list_by_billing_account(
            billing_account_name="str",
            api_version="2024-04-01",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...



import os
from typing import Optional, Dict


def str_to_bool(input_string: str) -> bool:
    """
    Takes a boolean string representation and returns a bool type value.
    """
    if isinstance(input_string, bool):
        return input_string
    elif input_string.lower() in ("true", "t", "1"):
        return True
    elif input_string.lower() in ("false", "f", "0"):
        return False
    else:
        return False


def discover_repo_root(input_repo: Optional[str] = None):
    """
    Resolves the root of the repository given a current working directory. This function should be used if a target repo argument is not provided.
    If the value of input_repo has value, that will supplant the path ascension logic.
    """

    if input_repo is not None:
        if os.path.exists(input_repo):
            return input_repo

    current_dir: str = os.getcwd()

    while current_dir is not None and not (os.path.dirname(current_dir) == current_dir):
        if os.path.exists(os.path.join(current_dir, ".git")):
            return current_dir
        else:
            current_dir = os.path.dirname(current_dir)

    raise Exception(
        "Commands invoked against azure-sdk-tooling should either be run from within the repo directory or provide --repo_root argument that directs at one."
    )


def get_artifact_directory(input_directory: Optional[str] = None) -> str:
    """
    Resolves the root of an artifact directory that the \"sdk_build\" action will output to!
    """
    if input_directory is not None:
        if not os.path.exists(input_directory):
            os.makedirs(input_directory)
        return input_directory
    return os.getenv("SDK_ARTIFACT_DIRECTORY", os.path.join(discover_repo_root(), ".artifacts"))


def get_log_directory(input_directory: Optional[str] = None) -> str:
    """
    Resolves the location of the log directory.
    """
    if input_directory is not None:
        if not os.path.exists(input_directory):
            os.makedirs(input_directory)
        return input_directory
    return os.getenv("SDK_LOG_DIRECTORY", os.path.join(discover_repo_root(), ".logs"))


def in_ci() -> int:
    # TF_BUILD is set to `true` on azure devops agents, returns 1
    # CI is set to `true` on github actions agents, return 2
    # 0 otherwise
    if os.getenv("TF_BUILD", None):
        return 1

    if os.getenv("CI", None):
        return 2

    return 0


def in_public() -> int:
    # Returns 3 if the build originates from a pull request
    # 0 otherwise
    if os.getenv("BUILD_REASON") == "PullRequest" or os.getenv("GITHUB_EVENT_NAME") == "pull_request":
        return 3

    return 0


def in_analyze_weekly() -> int:
    # Returns 4 if the build originates from the tests-weekly analyze job
    # 0 otherwise
    if (
        "tests-weekly" in os.getenv("SYSTEM_DEFINITIONNAME", "")
        and os.getenv("SYSTEM_STAGEDISPLAYNAME", "") == "Analyze_Test"
    ):
        return 4
    return 0


DEV_BUILD_IDENTIFIER = os.getenv("SDK_DEV_BUILD_IDENTIFIER", "a")
DEFAULT_BUILD_ID = os.getenv("GITHUB_RUN_ID", os.getenv("BUILD.BUILDID", os.getenv("SDK_BUILD_ID", "20220101.1")))

DEFAULT_ENVIRONMENT_VARIABLES = {
    "SPHINX_APIDOC_OPTIONS": "members,undoc-members,inherited-members",
    "VIRTUALENV_WHEEL": "0.45.1",
    "VIRTUALENV_PIP": "24.0",
    "VIRTUALENV_SETUPTOOLS": "75.3.2",
    "PIP_EXTRA_INDEX_URL": "https://pypi.python.org/simple",
    # I haven't spent much time looking to see if a variable exists when invoking uv run. there might be one already that we can depend
    # on for get_pip_command adjustment.
    "IN_UV": "1",
}


def set_environment_from_dictionary(settings: Dict[str, str]) -> None:
    """
    Sets default environment variables for any given process.
    Args:
        settings (Dict[str, str]): A dictionary of environment variable names and their default values.
    """
    for key, value in settings.items():
        os.environ.setdefault(key, value)


def set_envvar_defaults(settings: Optional[Dict[str, str]] = None) -> None:
    """
    Sets default environment variables for any given process to our default dictionary.
    Args:
        settings (Dict[str, str]): A dictionary of environment variable names and their default values.
    """
    set_environment_from_dictionary(DEFAULT_ENVIRONMENT_VARIABLES)

    if settings:
        # this will override any defaults set prior in the case of override
        set_environment_from_dictionary(settings)



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
from enum import Enum
import warnings
from typing import Mapping, Optional, Union, Any, cast
from typing_extensions import Literal, TypedDict, Protocol, runtime_checkable, NotRequired

from azure.core import CaseInsensitiveEnumMeta


class DeprecatedEnumMeta(CaseInsensitiveEnumMeta):

    def __getattribute__(cls, item):
        if item.upper() == "MICROSOFT_BOT":
            warnings.warn(
                "MICROSOFT_BOT is deprecated and has been replaced by \
                          MICROSOFT_TEAMS_APP identifier.",
                DeprecationWarning,
            )
            item = "MICROSOFT_TEAMS_APP"
        return super().__getattribute__(item)


class CommunicationIdentifierKind(str, Enum, metaclass=DeprecatedEnumMeta):
    """Communication Identifier Kind.

    For checking yet unknown identifiers it is better to rely on the presence of the `raw_id` property,
    as new or existing distinct type identifiers always contain the `raw_id` property.
    It is not advisable to rely on the `kind` property with a value `unknown`,
    as it could become a new or existing distinct type in the future.
    """

    UNKNOWN = "unknown"
    COMMUNICATION_USER = "communication_user"
    PHONE_NUMBER = "phone_number"
    MICROSOFT_TEAMS_USER = "microsoft_teams_user"
    MICROSOFT_TEAMS_APP = "microsoft_teams_app"
    TEAMS_EXTENSION_USER = "teams_extension_user"


class CommunicationCloudEnvironment(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The cloud environment that the identifier belongs to"""

    PUBLIC = "PUBLIC"
    DOD = "DOD"
    GCCH = "GCCH"


@runtime_checkable
class CommunicationIdentifier(Protocol):
    """Communication Identifier."""

    @property
    def raw_id(self) -> str:
        """The raw ID of the identifier."""
        ...

    @property
    def kind(self) -> CommunicationIdentifierKind:
        """The type of identifier."""
        ...

    @property
    def properties(self) -> Mapping[str, Any]:
        """The properties of the identifier."""
        ...


PHONE_NUMBER_PREFIX = "4:"
BOT_PREFIX = "28:"
BOT_PUBLIC_CLOUD_PREFIX = "28:orgid:"
BOT_DOD_CLOUD_PREFIX = "28:dod:"
BOT_DOD_CLOUD_GLOBAL_PREFIX = "28:dod-global:"
BOT_GCCH_CLOUD_PREFIX = "28:gcch:"
BOT_GCCH_CLOUD_GLOBAL_PREFIX = "28:gcch-global:"
TEAMS_APP_PUBLIC_CLOUD_PREFIX = "28:orgid:"
TEAMS_APP_DOD_CLOUD_PREFIX = "28:dod:"
TEAMS_APP_GCCH_CLOUD_PREFIX = "28:gcch:"
TEAMS_USER_ANONYMOUS_PREFIX = "8:teamsvisitor:"
TEAMS_USER_PUBLIC_CLOUD_PREFIX = "8:orgid:"
TEAMS_USER_DOD_CLOUD_PREFIX = "8:dod:"
TEAMS_USER_GCCH_CLOUD_PREFIX = "8:gcch:"
ACS_USER_PREFIX = "8:acs:"
ACS_USER_DOD_CLOUD_PREFIX = "8:dod-acs:"
ACS_USER_GCCH_CLOUD_PREFIX = "8:gcch-acs:"
SPOOL_USER_PREFIX = "8:spool:"

PHONE_NUMBER_ANONYMOUS_SUFFIX = "anonymous"


class CommunicationUserProperties(TypedDict):
    """Dictionary of properties for a CommunicationUserIdentifier."""

    id: str
    """ID of the Communication user as returned from Azure Communication Identity."""


class CommunicationUserIdentifier:
    """Represents a user in Azure Communication Service."""

    kind: Literal[CommunicationIdentifierKind.COMMUNICATION_USER] = CommunicationIdentifierKind.COMMUNICATION_USER
    """The type of identifier."""
    properties: CommunicationUserProperties
    """The properties of the identifier."""
    raw_id: str
    """The raw ID of the identifier."""

    def __init__(self, id: str, **kwargs: Any) -> None:
        """
        :param str id: ID of the Communication user as returned from Azure Communication Identity.
        :keyword str raw_id: The raw ID of the identifier. If not specified, the 'id' value will be used.
        """
        self.properties = CommunicationUserProperties(id=id)
        raw_id: Optional[str] = kwargs.get("raw_id")
        self.raw_id = raw_id if raw_id is not None else id

    def __eq__(self, other):
        try:
            if other.raw_id:
                return self.raw_id == other.raw_id
            return self.raw_id == other.properties["id"]
        except Exception:  # pylint: disable=broad-except
            return False


class PhoneNumberProperties(TypedDict):
    """Dictionary of properties for a PhoneNumberIdentifier."""

    value: str
    """The phone number in E.164 format."""
    asserted_id: NotRequired[str]
    """The asserted Id set on a phone number to distinguish from other connections made through the same number."""
    is_anonymous: NotRequired[bool]
    """True if the phone number is anonymous, e.g. when used to represent a hidden caller Id."""


class PhoneNumberIdentifier:
    """Represents a phone number."""

    kind: Literal[CommunicationIdentifierKind.PHONE_NUMBER] = CommunicationIdentifierKind.PHONE_NUMBER
    """The type of identifier."""
    properties: PhoneNumberProperties
    """The properties of the identifier."""
    raw_id: str
    """The raw ID of the identifier."""

    def __init__(self, value: str, **kwargs: Any) -> None:
        """
        :param str value: The phone number.
        :keyword str raw_id: The raw ID of the identifier. If not specified, this will be constructed from
          the 'value' parameter.
        """

        raw_id: Optional[str] = kwargs.get("raw_id")
        is_anonymous: bool

        if raw_id is not None:
            phone_number = raw_id[len(PHONE_NUMBER_PREFIX) :]
            is_anonymous = phone_number == PHONE_NUMBER_ANONYMOUS_SUFFIX
            asserted_id_index = -1 if is_anonymous else phone_number.rfind("_") + 1
            has_asserted_id = 0 < asserted_id_index < len(phone_number)
            if has_asserted_id:
                self.properties = PhoneNumberProperties(
                    value=value, is_anonymous=is_anonymous, asserted_id=phone_number[asserted_id_index:]
                )
            else:
                self.properties = PhoneNumberProperties(value=value, is_anonymous=is_anonymous)
        else:
            self.properties = PhoneNumberProperties(value=value)
        self.raw_id = raw_id if raw_id is not None else self._format_raw_id(self.properties)

    def __eq__(self, other):
        try:
            if other.raw_id:
                return self.raw_id == other.raw_id
            return self.raw_id == self._format_raw_id(other.properties)
        except Exception:  # pylint:disable=broad-except
            return False

    def _format_raw_id(self, properties: PhoneNumberProperties) -> str:
        # We just assume correct E.164 format here because
        # validation should only happen server-side, not client-side.
        value = properties["value"]
        return f"{PHONE_NUMBER_PREFIX}{value}"


class UnknownIdentifier:
    """Represents an identifier of an unknown type.

    It will be encountered in communications with endpoints that are not
    identifiable by this version of the SDK.

    For checking yet unknown identifiers it is better to rely on the presence of the `raw_id` property,
    as new or existing distinct type identifiers always contain the `raw_id` property.
    It is not advisable to rely on the `kind` property with a value `unknown`,
    as it could become a new or existing distinct type in the future.
    """

    kind: Literal[CommunicationIdentifierKind.UNKNOWN] = CommunicationIdentifierKind.UNKNOWN
    """The type of identifier."""
    properties: Mapping[str, Any]
    """The properties of the identifier."""
    raw_id: str
    """The raw ID of the identifier."""

    def __init__(self, identifier: str) -> None:
        """
        :param str identifier: The ID of the identifier.
        """
        self.raw_id = identifier
        self.properties = {}

    def __eq__(self, other):
        try:
            return self.raw_id == other.raw_id
        except AttributeError:
            return False


class MicrosoftTeamsUserProperties(TypedDict):
    """Dictionary of properties for a MicrosoftTeamsUserIdentifier."""

    user_id: str
    """The id of the Microsoft Teams user. If the user isn't anonymous, the id is the AAD object id of the user."""
    is_anonymous: bool
    """Set this to true if the user is anonymous for example when joining a meeting with a share link."""
    cloud: Union[CommunicationCloudEnvironment, str]
    """Cloud environment that this identifier belongs to."""


class MicrosoftTeamsUserIdentifier:
    """Represents an identifier for a Microsoft Teams user."""

    kind: Literal[CommunicationIdentifierKind.MICROSOFT_TEAMS_USER] = CommunicationIdentifierKind.MICROSOFT_TEAMS_USER
    """The type of identifier."""
    properties: MicrosoftTeamsUserProperties
    """The properties of the identifier."""
    raw_id: str
    """The raw ID of the identifier."""

    def __init__(self, user_id: str, **kwargs: Any) -> None:
        """
        :param str user_id: Microsoft Teams user id.
        :keyword bool is_anonymous: `True` if the identifier is anonymous. Default value is `False`.
        :keyword cloud: Cloud environment that the user belongs to. Default value is `PUBLIC`.
        :paramtype cloud: str or :class:`~.CommunicationCloudEnvironment`
        :keyword str raw_id: The raw ID of the identifier. If not specified, this value will be constructed from
         the other properties.
        """
        self.properties = MicrosoftTeamsUserProperties(
            user_id=user_id,
            is_anonymous=kwargs.get("is_anonymous", False),
            cloud=kwargs.get("cloud") or CommunicationCloudEnvironment.PUBLIC,
        )
        raw_id: Optional[str] = kwargs.get("raw_id")
        self.raw_id = raw_id if raw_id is not None else self._format_raw_id(self.properties)

    def __eq__(self, other):
        try:
            if other.raw_id:
                return self.raw_id == other.raw_id
            return self.raw_id == self._format_raw_id(other.properties)
        except Exception:  # pylint: disable=broad-except
            return False

    def _format_raw_id(self, properties: MicrosoftTeamsUserProperties) -> str:
        user_id = properties["user_id"]
        if properties["is_anonymous"]:
            return f"{TEAMS_USER_ANONYMOUS_PREFIX}{user_id}"
        cloud = properties["cloud"]
        if cloud == CommunicationCloudEnvironment.DOD:
            return f"{TEAMS_USER_DOD_CLOUD_PREFIX}{user_id}"
        if cloud == CommunicationCloudEnvironment.GCCH:
            return f"{TEAMS_USER_GCCH_CLOUD_PREFIX}{user_id}"
        if cloud == CommunicationCloudEnvironment.PUBLIC:
            return f"{TEAMS_USER_PUBLIC_CLOUD_PREFIX}{user_id}"
        return f"{TEAMS_USER_PUBLIC_CLOUD_PREFIX}{user_id}"


class MicrosoftTeamsAppProperties(TypedDict):
    """Dictionary of properties for a MicrosoftTeamsAppIdentifier."""

    app_id: str
    """The id of the Microsoft Teams application."""
    cloud: Union[CommunicationCloudEnvironment, str]
    """Cloud environment that this identifier belongs to."""


class _botbackcompatdict(dict):
    """Backwards compatible properties."""

    def __getitem__(self, __key: Any) -> Any:
        try:
            return super().__getitem__(__key)
        except KeyError:
            if __key == "bot_id":
                return super().__getitem__("app_id")
            if __key == "is_resource_account_configured":
                return True
            raise


class MicrosoftTeamsAppIdentifier:
    """Represents an identifier for a Microsoft Teams application."""

    kind: Literal[CommunicationIdentifierKind.MICROSOFT_TEAMS_APP] = CommunicationIdentifierKind.MICROSOFT_TEAMS_APP
    """The type of identifier."""
    properties: MicrosoftTeamsAppProperties
    """The properties of the identifier."""
    raw_id: str
    """The raw ID of the identifier."""

    def __init__(self, app_id: str, **kwargs: Any) -> None:
        """
        :param str app_id: Microsoft Teams application id.
        :keyword cloud: Cloud environment that the application belongs to. Default value is `PUBLIC`.
        :paramtype cloud: str or :class:`~.CommunicationCloudEnvironment`
        :keyword str raw_id: The raw ID of the identifier. If not specified, this value will be constructed
         from the other properties.
        """
        self.properties = cast(
            MicrosoftTeamsAppProperties,
            _botbackcompatdict(
                app_id=app_id,
                cloud=kwargs.get("cloud") or CommunicationCloudEnvironment.PUBLIC,
            ),
        )
        raw_id: Optional[str] = kwargs.get("raw_id")
        self.raw_id = raw_id if raw_id is not None else self._format_raw_id(self.properties)

    def __eq__(self, other):
        try:
            if other.raw_id:
                return self.raw_id == other.raw_id
            return self.raw_id == self._format_raw_id(other.properties)
        except Exception:  # pylint: disable=broad-except
            return False

    def _format_raw_id(self, properties: MicrosoftTeamsAppProperties) -> str:
        app_id = properties["app_id"]
        cloud = properties["cloud"]
        if cloud == CommunicationCloudEnvironment.DOD:
            return f"{TEAMS_APP_DOD_CLOUD_PREFIX}{app_id}"
        if cloud == CommunicationCloudEnvironment.GCCH:
            return f"{TEAMS_APP_GCCH_CLOUD_PREFIX}{app_id}"
        return f"{TEAMS_APP_PUBLIC_CLOUD_PREFIX}{app_id}"


class _MicrosoftBotIdentifier(MicrosoftTeamsAppIdentifier):
    """Represents an identifier for a Microsoft bot.

    DEPRECATED. Only used in cases of backwards compatibility.
    """

    def __init__(self, bot_id, **kwargs):
        """
        :param str bot_id: Microsoft bot id.
        :keyword bool is_resource_account_configured: `False` if the identifier is global.
         Default value is `True` for tennantzed bots.
        :keyword cloud: Cloud environment that the bot belongs to. Default value is `PUBLIC`.
        :paramtype cloud: str or :class:`~.CommunicationCloudEnvironment`
        """
        warnings.warn(
            "The MicrosoftBotIdentifier is deprecated and has been replaced by MicrosoftTeamsAppIdentifier.",
            DeprecationWarning,
        )
        super().__init__(bot_id, **kwargs)


class TeamsExtensionUserProperties(TypedDict):
    """Dictionary of properties for a TeamsExtensionUserIdentifier."""

    user_id: str
    """The id of the Teams extension user."""
    tenant_id: str
    """The tenant id associated with the user."""
    resource_id: str
    """The Communication Services resource id."""
    cloud: Union[CommunicationCloudEnvironment, str]
    """Cloud environment that this identifier belongs to."""


class TeamsExtensionUserIdentifier:
    """Represents an identifier for a Teams Extension user."""

    kind: Literal[CommunicationIdentifierKind.TEAMS_EXTENSION_USER] = CommunicationIdentifierKind.TEAMS_EXTENSION_USER
    """The type of identifier."""
    properties: TeamsExtensionUserProperties
    """The properties of the identifier."""
    raw_id: str
    """The raw ID of the identifier."""

    def __init__(self, *, user_id: str, tenant_id: str, resource_id: str, **kwargs: Any) -> None:
        """
        :param str user_id: Teams extension user id.
        :param str tenant_id: Tenant id associated with the user.
        :param str resource_id: The Communication Services resource id.
        :keyword cloud: Cloud environment that the user belongs to. Default value is `PUBLIC`.
        :paramtype cloud: str or :class:`~.CommunicationCloudEnvironment`
        :keyword str raw_id: The raw ID of the identifier.
         If not specified, this value will be constructed from the other properties.
        """
        self.properties = TeamsExtensionUserProperties(
            user_id=user_id,
            tenant_id=tenant_id,
            resource_id=resource_id,
            cloud=kwargs.get("cloud") or CommunicationCloudEnvironment.PUBLIC,
        )
        raw_id: Optional[str] = kwargs.get("raw_id")
        self.raw_id = raw_id if raw_id is not None else self._format_raw_id(self.properties)

    def __eq__(self, other):
        try:
            if other.raw_id:
                return self.raw_id == other.raw_id
            return self.raw_id == self._format_raw_id(other.properties)
        except Exception:  # pylint: disable=broad-except
            return False

    def _format_raw_id(self, properties: TeamsExtensionUserProperties) -> str:
        # The prefix depends on the cloud
        cloud = properties["cloud"]
        if cloud == CommunicationCloudEnvironment.DOD:
            prefix = ACS_USER_DOD_CLOUD_PREFIX
        elif cloud == CommunicationCloudEnvironment.GCCH:
            prefix = ACS_USER_GCCH_CLOUD_PREFIX
        else:
            prefix = ACS_USER_PREFIX
        return f"{prefix}{properties['resource_id']}_{properties['tenant_id']}_{properties['user_id']}"


def try_create_teams_extension_user(prefix: str, suffix: str) -> Optional[TeamsExtensionUserIdentifier]:
    segments = suffix.split("_")
    if len(segments) != 3:
        return None
    resource_id, tenant_id, user_id = segments
    if prefix == ACS_USER_PREFIX:
        cloud = CommunicationCloudEnvironment.PUBLIC
    elif prefix == ACS_USER_DOD_CLOUD_PREFIX:
        cloud = CommunicationCloudEnvironment.DOD
    elif prefix == ACS_USER_GCCH_CLOUD_PREFIX:
        cloud = CommunicationCloudEnvironment.GCCH
    else:
        raise ValueError("Invalid MRI")
    return TeamsExtensionUserIdentifier(user_id=user_id, tenant_id=tenant_id, resource_id=resource_id, cloud=cloud)


def identifier_from_raw_id(raw_id: str) -> CommunicationIdentifier:  # pylint: disable=too-many-return-statements
    """
    Creates a CommunicationIdentifier from a given raw ID.

    When storing raw IDs use this function to restore the identifier that was encoded in the raw ID.

    :param str raw_id: A raw ID to construct the CommunicationIdentifier from.
    :return: The CommunicationIdentifier parsed from the raw_id.
    :rtype: :class:`~.CommunicationIdentifier`
    """
    if raw_id.startswith(PHONE_NUMBER_PREFIX):
        return PhoneNumberIdentifier(value=raw_id[len(PHONE_NUMBER_PREFIX) :], raw_id=raw_id)

    segments = raw_id.split(":", maxsplit=2)
    if len(segments) < 3:
        return UnknownIdentifier(identifier=raw_id)

    prefix = f"{segments[0]}:{segments[1]}:"
    suffix = segments[2]
    if prefix == TEAMS_USER_ANONYMOUS_PREFIX:
        return MicrosoftTeamsUserIdentifier(user_id=suffix, is_anonymous=True, raw_id=raw_id)
    if prefix == TEAMS_USER_PUBLIC_CLOUD_PREFIX:
        return MicrosoftTeamsUserIdentifier(
            user_id=suffix,
            is_anonymous=False,
            cloud=CommunicationCloudEnvironment.PUBLIC,
            raw_id=raw_id,
        )
    if prefix == TEAMS_USER_DOD_CLOUD_PREFIX:
        return MicrosoftTeamsUserIdentifier(
            user_id=suffix,
            is_anonymous=False,
            cloud=CommunicationCloudEnvironment.DOD,
            raw_id=raw_id,
        )
    if prefix == TEAMS_USER_GCCH_CLOUD_PREFIX:
        return MicrosoftTeamsUserIdentifier(
            user_id=suffix,
            is_anonymous=False,
            cloud=CommunicationCloudEnvironment.GCCH,
            raw_id=raw_id,
        )
    if prefix == TEAMS_APP_PUBLIC_CLOUD_PREFIX:
        return MicrosoftTeamsAppIdentifier(
            app_id=suffix,
            cloud=CommunicationCloudEnvironment.PUBLIC,
            raw_id=raw_id,
        )
    if prefix == TEAMS_APP_DOD_CLOUD_PREFIX:
        return MicrosoftTeamsAppIdentifier(
            app_id=suffix,
            cloud=CommunicationCloudEnvironment.DOD,
            raw_id=raw_id,
        )
    if prefix == TEAMS_APP_GCCH_CLOUD_PREFIX:
        return MicrosoftTeamsAppIdentifier(
            app_id=suffix,
            cloud=CommunicationCloudEnvironment.GCCH,
            raw_id=raw_id,
        )
    if prefix == SPOOL_USER_PREFIX:
        return CommunicationUserIdentifier(id=raw_id, raw_id=raw_id)

    if prefix in [
        ACS_USER_PREFIX,
        ACS_USER_DOD_CLOUD_PREFIX,
        ACS_USER_GCCH_CLOUD_PREFIX,
    ]:
        identifier = try_create_teams_extension_user(prefix, suffix)
        if identifier is not None:
            return identifier
        return CommunicationUserIdentifier(id=raw_id, raw_id=raw_id)
    return UnknownIdentifier(identifier=raw_id)
